<?php

namespace Drupal\Tests\visitors\Unit\Controller\Report;

use Drupal\Core\Datetime\DateFormatterInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Form\FormBuilderInterface;
use Drupal\Tests\UnitTestCase;
use Drupal\visitors\Controller\Report\Node;
use Drupal\visitors\VisitorsReportInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Unit tests for the Node controller.
 *
 * @group visitors
 */
class NodeControllerTest extends UnitTestCase {

  /**
   * The mocked Date Formatter service.
   *
   * @var \Drupal\Core\Datetime\DateFormatterInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $dateFormatter;

  /**
   * The mocked Form Builder service.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $formBuilder;

  /**
   * The mocked Database Connection service.
   *
   * @var \Drupal\Core\Database\Connection|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $database;

  /**
   * The mocked Module Handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $moduleHandler;

  /**
   * The mocked Request Stack service.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $requestStack;

  /**
   * The mocked Visitors Report service.
   *
   * @var \Drupal\visitors\VisitorsReportInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $visitorsReport;

  /**
   * The Node controller under test.
   *
   * @var \Drupal\visitors\Controller\Report\Node
   */
  protected $controller;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    if (!defined('RESPONSIVE_PRIORITY_LOW')) {
      define('RESPONSIVE_PRIORITY_LOW', 'priority-low');
    }

    $container = new ContainerBuilder();

    $this->dateFormatter = $this->createMock(DateFormatterInterface::class);
    $container->set('date.formatter', $this->dateFormatter);

    $this->formBuilder = $this->createMock(FormBuilderInterface::class);
    $container->set('form_builder', $this->formBuilder);

    $this->moduleHandler = $this->createMock(ModuleHandlerInterface::class);
    $container->set('module_handler', $this->moduleHandler);

    $this->visitorsReport = $this->createMock(VisitorsReportInterface::class);
    $container->set('visitors.report', $this->visitorsReport);

    $string_translation = $this->getStringTranslationStub();
    $container->set('string_translation', $string_translation);

    $this->controller = Node::create($container);
  }

  /**
   * Tests the display() method with a valid node.
   */
  public function testDisplayWithValidNode(): void {
    $node = $this->createMock('Drupal\Core\Entity\EntityInterface');
    $node->expects($this->once())
      ->method('id')
      ->willReturn(123);

    $this->moduleHandler
      ->expects($this->once())
      ->method('moduleExists')
      ->with('node')
      ->willReturn(TRUE);

    $rows = [
      [
        '#markup' => 'Row 1',
      ],
      [
        '#markup' => 'Row 2',
      ],
    ];

    $this->visitorsReport->expects($this->once())
      ->method('node')
      ->willReturn($rows);

    // Invoke the display method and assert the expected render array.
    $renderArray = $this->controller->display($node);

    $this->assertArrayHasKey('visitors_date_filter_form', $renderArray);
    $this->assertArrayHasKey('visitors_table', $renderArray);
    $this->assertArrayHasKey('visitors_pager', $renderArray);
  }

  /**
   * Tests the display() method with an invalid node.
   */
  public function testDisplayWithInvalidNode(): void {
    $node = $this->createMock('Drupal\Core\Entity\EntityInterface');
    $this->moduleHandler->expects($this->once())
      ->method('moduleExists')
      ->with('node')
      ->willReturn(FALSE);

    $this->expectException(NotFoundHttpException::class);
    $this->controller->display($node);
  }

}
